##
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.
##
import warnings
warnings.warn(
    "The azure.quantum.plugins package will be deprecated. \
Please use azure.quantum.cirq and azure.quantum.qiskit instead.")
##
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.
##

from azure.quantum.cirq import *